@@ -6,15 +6,10 @@
 # full list see the documentation:
 # http://www.sphinx-doc.org/en/master/config
 
-import os
-import sys
-
-sys.path.insert(0, os.path.abspath('../..'))
 
 # -- Project information ------------------------------------------------------
 
 # General information about the project.
-project = 'oslo.messaging'
 copyright = '2018, Oslo Contributors'
 
 # -- General configuration ----------------------------------------------------
@@ -23,6 +18,7 @@ copyright = '2018, Oslo Contributors'
 # extensions coming with Sphinx (named 'sphinx.ext.*') or your custom ones.
 extensions = [
     'sphinx.ext.autodoc',
+    'sphinx.ext.todo',
     'openstackdocstheme',
     'stevedore.sphinxext',
     'oslo_config.sphinxext',
@@ -33,9 +29,6 @@ repository_name = 'openstack/oslo.messaging'
 bug_project = 'oslo.messaging'
 bug_tag = ''
 
-# Must set this variable to include year, month, day, hours, and minutes.
-html_last_updated_fmt = '%Y-%m-%d %H:%M'
-
 # The master toctree document.
 master_doc = 'index'
 
