@@ -19,6 +19,7 @@ import time
 import uuid
 
 import cachetools
+from oslo_utils import eventletutils
 from oslo_utils import timeutils
 from six import moves
 
@@ -44,11 +45,12 @@ class MessageOperationsHandler(object):
     serialized and run in the same thread, since underlying drivers like kombu
     are not thread safe.
     """
+
     def __init__(self, name):
         self.name = "%s (%s)" % (name, hex(id(self)))
         self._tasks = moves.queue.Queue()
 
-        self._shutdown = threading.Event()
+        self._shutdown = eventletutils.Event()
         self._shutdown_thread = threading.Thread(
             target=self._process_in_background)
         self._shutdown_thread.daemon = True
@@ -269,8 +271,8 @@ class AMQPListener(base.PollStyleListener):
         self.conn = conn
         self.msg_id_cache = rpc_amqp._MsgIdCache()
         self.incoming = []
-        self._shutdown = threading.Event()
-        self._shutoff = threading.Event()
+        self._shutdown = eventletutils.Event()
+        self._shutoff = eventletutils.Event()
         self._obsolete_reply_queues = ObsoleteReplyQueuesCache()
         self._message_operations_handler = MessageOperationsHandler(
             "AMQPListener")
@@ -303,8 +305,11 @@ class AMQPListener(base.PollStyleListener):
 
         while not self._shutdown.is_set():
             self._message_operations_handler.process()
+            LOG.debug("Listener is running")
 
             if self.incoming:
+                LOG.debug("Poll the incoming message with unique_id: %s",
+                          self.incoming[0].unique_id)
                 return self.incoming.pop(0)
 
             left = stopwatch.leftover(return_none=True)
@@ -314,8 +319,10 @@ class AMQPListener(base.PollStyleListener):
                 return None
 
             try:
+                LOG.debug("AMQPListener connection consume")
                 self.conn.consume(timeout=min(self._current_timeout, left))
             except rpc_common.Timeout:
+                LOG.debug("AMQPListener connection timeout")
                 self._current_timeout = max(self._current_timeout * 2,
                                             ACK_REQUEUE_EVERY_SECONDS_MAX)
             else:
@@ -323,8 +330,11 @@ class AMQPListener(base.PollStyleListener):
 
         # NOTE(sileht): listener is stopped, just processes remaining messages
         # and operations
+        LOG.debug("Listener is stopped")
         self._message_operations_handler.process()
         if self.incoming:
+            LOG.debug("Poll the incoming message with unique_id: %s",
+                      self.incoming[0].unique_id)
             return self.incoming.pop(0)
 
         self._shutoff.set()
@@ -370,7 +380,7 @@ class RpcAMQPListener(AMQPListener):
         try:
             message.acknowledge()
         except Exception as exc:
-            LOG.warning("Discarding RPC request due to failed acknowlege: %s",
+            LOG.warning("Discarding RPC request due to failed acknowledge: %s",
                         exc)
         else:
             # NOTE(kgiusti): be aware that even if the acknowledge call
@@ -386,8 +396,6 @@ class NotificationAMQPListener(AMQPListener):
 
 class ReplyWaiters(object):
 
-    WAKE_UP = object()
-
     def __init__(self):
         self._queues = {}
         self._wrn_threshold = 10
@@ -435,7 +443,7 @@ class ReplyWaiter(object):
 
         self.conn.declare_direct_consumer(reply_q, self)
 
-        self._thread_exit_event = threading.Event()
+        self._thread_exit_event = eventletutils.Event()
         self._thread = threading.Thread(target=self.poll)
         self._thread.daemon = True
         self._thread.start()
@@ -580,7 +588,7 @@ class AMQPDriverBase(base.BaseDriver):
 
     def _send(self, target, ctxt, message,
               wait_for_reply=None, timeout=None, call_monitor_timeout=None,
-              envelope=True, notify=False, retry=None):
+              envelope=True, notify=False, retry=None, transport_options=None):
 
         msg = message
 
@@ -626,7 +634,8 @@ class AMQPDriverBase(base.BaseDriver):
                               " topic '%(topic)s'", {'exchange': exchange,
                                                      'topic': topic})
                     conn.topic_send(exchange_name=exchange, topic=topic,
-                                    msg=msg, timeout=timeout, retry=retry)
+                                    msg=msg, timeout=timeout, retry=retry,
+                                    transport_options=transport_options)
 
             if wait_for_reply:
                 result = self._waiter.wait(msg_id, timeout,
@@ -639,9 +648,10 @@ class AMQPDriverBase(base.BaseDriver):
                 self._waiter.unlisten(msg_id)
 
     def send(self, target, ctxt, message, wait_for_reply=None, timeout=None,
-             call_monitor_timeout=None, retry=None):
+             call_monitor_timeout=None, retry=None, transport_options=None):
         return self._send(target, ctxt, message, wait_for_reply, timeout,
-                          call_monitor_timeout, retry=retry)
+                          call_monitor_timeout, retry=retry,
+                          transport_options=transport_options)
 
     def send_notification(self, target, ctxt, message, version, retry=None):
         return self._send(target, ctxt, message,
