@@ -27,6 +27,7 @@ import threading
 import time
 import uuid
 
+from oslo_utils import eventletutils
 from oslo_utils import importutils
 from six import moves
 from string import Template
@@ -51,8 +52,8 @@ if pyngus:
 # The Cyrus-based SASL tests can only be run if the installed version of proton
 # has been built with Cyrus SASL support.
 _proton = importutils.try_import("proton")
-CYRUS_ENABLED = (pyngus and pyngus.VERSION >= (2, 0, 0) and _proton
-                 and getattr(_proton.SASL, "extended", lambda: False)())
+CYRUS_ENABLED = (pyngus and pyngus.VERSION >= (2, 0, 0) and _proton and
+                 getattr(_proton.SASL, "extended", lambda: False)())
 # same with SSL
 # SSL_ENABLED = (_proton and getattr(_proton.SSL, "present", lambda: False)())
 SSL_ENABLED = False
@@ -75,19 +76,19 @@ class _ListenerThread(threading.Thread):
         self._msg_ack = msg_ack
         self.messages = moves.queue.Queue()
         self.daemon = True
-        self.started = threading.Event()
-        self._done = False
+        self.started = eventletutils.Event()
+        self._done = eventletutils.Event()
         self.start()
         self.started.wait()
 
     def run(self):
         LOG.debug("Listener started")
         self.started.set()
-        while not self._done:
+        while not self._done.is_set():
             for in_msg in self.listener.poll(timeout=0.5):
                 self.messages.put(in_msg)
                 self.msg_count -= 1
-                self._done = self.msg_count == 0
+                self.msg_count == 0 and self._done.set()
                 if self._msg_ack:
                     in_msg.acknowledge()
                     if in_msg.message.get('method') == 'echo':
@@ -110,16 +111,65 @@ class _ListenerThread(threading.Thread):
         return msgs
 
     def kill(self, timeout=30):
-        self._done = True
+        self._done.set()
         self.join(timeout)
 
 
+class _SlowResponder(_ListenerThread):
+    # an RPC listener that pauses delay seconds before replying
+    def __init__(self, listener, delay, msg_count=1):
+        self._delay = delay
+        super(_SlowResponder, self).__init__(listener, msg_count)
+
+    def run(self):
+        LOG.debug("_SlowResponder started")
+        self.started.set()
+        while not self._done.is_set():
+            for in_msg in self.listener.poll(timeout=0.5):
+                time.sleep(self._delay)
+                in_msg.acknowledge()
+                in_msg.reply(reply={'correlation-id':
+                                    in_msg.message.get('id')})
+                self.messages.put(in_msg)
+                self.msg_count -= 1
+                self.msg_count == 0 and self._done.set()
+
+
+class _CallMonitor(_ListenerThread):
+    # an RPC listener that generates heartbeats before
+    # replying.
+    def __init__(self, listener, delay, hb_count, msg_count=1):
+        self._delay = delay
+        self._hb_count = hb_count
+        super(_CallMonitor, self).__init__(listener, msg_count)
+
+    def run(self):
+        LOG.debug("_CallMonitor started")
+        self.started.set()
+        while not self._done.is_set():
+            for in_msg in self.listener.poll(timeout=0.5):
+                hb_rate = in_msg.client_timeout / 2.0
+                deadline = time.time() + self._delay
+                while deadline > time.time():
+                    if self._done.wait(hb_rate):
+                        return
+                    if self._hb_count > 0:
+                        in_msg.heartbeat()
+                        self._hb_count -= 1
+                in_msg.acknowledge()
+                in_msg.reply(reply={'correlation-id':
+                                    in_msg.message.get('id')})
+                self.messages.put(in_msg)
+                self.msg_count -= 1
+                self.msg_count == 0 and self._done.set()
+
+
 @testtools.skipUnless(pyngus, "proton modules not present")
 class TestProtonDriverLoad(test_utils.BaseTestCase):
 
     def setUp(self):
         super(TestProtonDriverLoad, self).setUp()
-        self.messaging_conf.transport_driver = 'amqp'
+        self.messaging_conf.transport_url = 'amqp://'
 
     def test_driver_load(self):
         transport = oslo_messaging.get_transport(self.conf)
@@ -365,27 +415,11 @@ class TestAmqpSend(_AmqpBrokerTestCaseAuto):
 
     def test_call_late_reply(self):
         """What happens if reply arrives after timeout?"""
-
-        class _SlowResponder(_ListenerThread):
-            def __init__(self, listener, delay):
-                self._delay = delay
-                super(_SlowResponder, self).__init__(listener, 1)
-
-            def run(self):
-                self.started.set()
-                while not self._done:
-                    for in_msg in self.listener.poll(timeout=0.5):
-                        time.sleep(self._delay)
-                        in_msg.acknowledge()
-                        in_msg.reply(reply={'correlation-id':
-                                            in_msg.message.get('id')})
-                        self.messages.put(in_msg)
-                        self._done = True
-
         driver = amqp_driver.ProtonDriver(self.conf, self._broker_url)
         target = oslo_messaging.Target(topic="test-topic")
         listener = _SlowResponder(
-            driver.listen(target, None, None)._poll_style_listener, 3)
+            driver.listen(target, None, None)._poll_style_listener,
+            delay=3)
 
         self.assertRaises(oslo_messaging.MessagingTimeout,
                           driver.send, target,
@@ -410,14 +444,14 @@ class TestAmqpSend(_AmqpBrokerTestCaseAuto):
 
             def run(self):
                 self.started.set()
-                while not self._done:
+                while not self._done.is_set():
                     for in_msg in self.listener.poll(timeout=0.5):
                         try:
                             raise RuntimeError("Oopsie!")
                         except RuntimeError:
                             in_msg.reply(reply=None,
                                          failure=sys.exc_info())
-                        self._done = True
+                        self._done.set()
 
         driver = amqp_driver.ProtonDriver(self.conf, self._broker_url)
         target = oslo_messaging.Target(topic="test-topic")
@@ -442,13 +476,13 @@ class TestAmqpSend(_AmqpBrokerTestCaseAuto):
 
             def run(self):
                 self.started.set()
-                while not self._done:
+                while not self._done.is_set():
                     for in_msg in self.listener.poll(timeout=0.5):
                         # reply will never be acked (simulate drop):
                         in_msg._reply_to = "!no-ack!"
                         in_msg.reply(reply={'correlation-id':
                                             in_msg.message.get("id")})
-                        self._done = True
+                        self._done.set()
 
         driver = amqp_driver.ProtonDriver(self.conf, self._broker_url)
         driver._default_reply_timeout = 1
@@ -555,6 +589,66 @@ class TestAmqpSend(_AmqpBrokerTestCaseAuto):
 
         driver.cleanup()
 
+    def test_call_monitor_ok(self):
+        # verify keepalive by delaying the reply > heartbeat interval
+        driver = amqp_driver.ProtonDriver(self.conf, self._broker_url)
+        target = oslo_messaging.Target(topic="test-topic")
+        listener = _CallMonitor(
+            driver.listen(target, None, None)._poll_style_listener,
+            delay=11,
+            hb_count=100)
+        rc = driver.send(target,
+                         {"context": True},
+                         {"method": "echo", "id": "1"},
+                         wait_for_reply=True,
+                         timeout=60,
+                         call_monitor_timeout=5)
+        self.assertIsNotNone(rc)
+        self.assertEqual("1", rc.get('correlation-id'))
+        listener.join(timeout=30)
+        self.assertFalse(listener.isAlive())
+        driver.cleanup()
+
+    def test_call_monitor_bad_no_heartbeat(self):
+        # verify call fails if keepalives stop coming
+        driver = amqp_driver.ProtonDriver(self.conf, self._broker_url)
+        target = oslo_messaging.Target(topic="test-topic")
+        listener = _CallMonitor(
+            driver.listen(target, None, None)._poll_style_listener,
+            delay=11,
+            hb_count=1)
+        self.assertRaises(oslo_messaging.MessagingTimeout,
+                          driver.send,
+                          target,
+                          {"context": True},
+                          {"method": "echo", "id": "1"},
+                          wait_for_reply=True,
+                          timeout=60,
+                          call_monitor_timeout=5)
+        listener.kill()
+        self.assertFalse(listener.isAlive())
+        driver.cleanup()
+
+    def test_call_monitor_bad_call_timeout(self):
+        # verify call fails if deadline hit regardless of heartbeat activity
+        driver = amqp_driver.ProtonDriver(self.conf, self._broker_url)
+        target = oslo_messaging.Target(topic="test-topic")
+        listener = _CallMonitor(
+            driver.listen(target, None, None)._poll_style_listener,
+            delay=20,
+            hb_count=100)
+        self.assertRaises(oslo_messaging.MessagingTimeout,
+                          driver.send,
+                          target,
+                          {"context": True},
+                          {"method": "echo", "id": "1"},
+                          wait_for_reply=True,
+                          timeout=11,
+                          call_monitor_timeout=5)
+        listener.kill()
+        self.assertFalse(listener.isAlive())
+        driver.cleanup()
+
 
 class TestAmqpNotification(_AmqpBrokerTestCaseAuto):
     """Test sending and receiving notifications."""
@@ -750,7 +844,7 @@ mech_list: ${mechs}
                                   sasl_config_dir=_dir,
                                   sasl_config_name="openstack")
         self._broker.start()
-        self.messaging_conf.transport_driver = 'amqp'
+        self.messaging_conf.transport_url = 'amqp://'
         self.conf = self.messaging_conf.conf
 
     def tearDown(self):
@@ -811,17 +905,6 @@ mech_list: ${mechs}
                           addr,
                           retry=0)
 
-    def test_authentication_default_username(self):
-        """Verify that a configured username/password is used if none appears
-        in the URL.
-        Deprecated: username password deprecated in favor of transport_url
-        """
-        addr = "amqp://%s:%d" % (self._broker.host, self._broker.port)
-        self.config(username="joe@myrealm",
-                    password="secret",
-                    group="oslo_messaging_amqp")
-        self._authentication_test(addr)
-
     def test_authentication_default_realm(self):
         """Verify that default realm is used if none present in username"""
         addr = "amqp://joe:secret@%s:%d" % (self._broker.host,
@@ -1064,7 +1147,7 @@ class TestLinkRecovery(_AmqpBrokerTestCase):
         self._addrs = {'unicast.test-topic': 2,
                        'broadcast.test-topic.all': 2,
                        'exclusive.test-topic.server': 2}
-        self._recovered = threading.Event()
+        self._recovered = eventletutils.Event()
         self._count = 0
 
         def _on_active(link):
@@ -1223,10 +1306,10 @@ class TestAddressing(test_utils.BaseTestCase):
         s1_payload = [m.message.get('msg') for m in rl[0].get_messages()]
         s2_payload = [m.message.get('msg') for m in rl[1].get_messages()]
 
-        self.assertTrue("Server1" in s1_payload
-                        and "Server2" not in s1_payload)
-        self.assertTrue("Server2" in s2_payload
-                        and "Server1" not in s2_payload)
+        self.assertTrue("Server1" in s1_payload and
+                        "Server2" not in s1_payload)
+        self.assertTrue("Server2" in s2_payload and
+                        "Server1" not in s2_payload)
         self.assertEqual(s1_payload.count("Fanout"), 1)
         self.assertEqual(s2_payload.count("Fanout"), 1)
         self.assertEqual((s1_payload + s2_payload).count("Anycast1"), 1)
@@ -2018,7 +2101,7 @@ class FakeBroker(threading.Thread):
 
         self._connections = {}
         self._sources = {}
-        self._pause = threading.Event()
+        self._pause = eventletutils.Event()
         # count of messages forwarded, by messaging pattern
         self.direct_count = 0
         self.topic_count = 0
