@@ -108,11 +108,15 @@ class TestKafkaDriver(test_utils.BaseTestCase):
                 'bootstrap.servers': '',
                 'linger.ms': mock.ANY,
                 'batch.num.messages': mock.ANY,
+                'compression.codec': 'none',
                 'security.protocol': 'PLAINTEXT',
                 'sasl.mechanism': 'PLAIN',
                 'sasl.username': mock.ANY,
                 'sasl.password': mock.ANY,
-                'ssl.ca.location': ''
+                'ssl.ca.location': '',
+                'ssl.certificate.location': '',
+                'ssl.key.location': '',
+                'ssl.key.password': '',
             })
 
     def test_listen(self):
@@ -138,6 +142,9 @@ class TestKafkaDriver(test_utils.BaseTestCase):
                 'sasl.username': mock.ANY,
                 'sasl.password': mock.ANY,
                 'ssl.ca.location': '',
+                'ssl.certificate.location': '',
+                'ssl.key.location': '',
+                'ssl.key.password': '',
                 'default.topic.config': {'auto.offset.reset': 'latest'}
             })
 
