@@ -1026,26 +1026,26 @@ class ConnectionLockTestCase(test_utils.BaseTestCase):
         return get_elapsed_time
 
     def test_workers_only(self):
-        l = rabbit_driver.ConnectionLock()
-        t1 = self._thread(l, 1)
-        t2 = self._thread(l, 1)
+        lock = rabbit_driver.ConnectionLock()
+        t1 = self._thread(lock, 1)
+        t2 = self._thread(lock, 1)
         self.assertAlmostEqual(1, t1(), places=0)
         self.assertAlmostEqual(2, t2(), places=0)
 
     def test_worker_and_heartbeat(self):
-        l = rabbit_driver.ConnectionLock()
-        t1 = self._thread(l, 1)
-        t2 = self._thread(l, 1, heartbeat=True)
+        lock = rabbit_driver.ConnectionLock()
+        t1 = self._thread(lock, 1)
+        t2 = self._thread(lock, 1, heartbeat=True)
         self.assertAlmostEqual(1, t1(), places=0)
         self.assertAlmostEqual(2, t2(), places=0)
 
     def test_workers_and_heartbeat(self):
-        l = rabbit_driver.ConnectionLock()
-        t1 = self._thread(l, 1)
-        t2 = self._thread(l, 1)
-        t3 = self._thread(l, 1)
-        t4 = self._thread(l, 1, heartbeat=True)
-        t5 = self._thread(l, 1)
+        lock = rabbit_driver.ConnectionLock()
+        t1 = self._thread(lock, 1)
+        t2 = self._thread(lock, 1)
+        t3 = self._thread(lock, 1)
+        t4 = self._thread(lock, 1, heartbeat=True)
+        t5 = self._thread(lock, 1)
         self.assertAlmostEqual(1, t1(), places=0)
         self.assertAlmostEqual(2, t4(), places=0)
         self.assertAlmostEqual(3, t2(), places=0)
@@ -1053,8 +1053,8 @@ class ConnectionLockTestCase(test_utils.BaseTestCase):
         self.assertAlmostEqual(5, t5(), places=0)
 
     def test_heartbeat(self):
-        l = rabbit_driver.ConnectionLock()
-        t1 = self._thread(l, 1, heartbeat=True)
-        t2 = self._thread(l, 1)
+        lock = rabbit_driver.ConnectionLock()
+        t1 = self._thread(lock, 1, heartbeat=True)
+        t2 = self._thread(lock, 1)
         self.assertAlmostEqual(1, t1(), places=0)
         self.assertAlmostEqual(2, t2(), places=0)
