@@ -14,11 +14,11 @@
 #    under the License.
 
 import threading
-import warnings
 
 import eventlet
 import fixtures
 from oslo_config import cfg
+from oslo_utils import eventletutils
 from six.moves import mock
 import testscenarios
 
@@ -35,9 +35,11 @@ load_tests = testscenarios.load_tests_apply_scenarios
 class ServerSetupMixin(object):
 
     class Server(object):
-        def __init__(self, transport, topic, server, endpoint, serializer):
+        def __init__(self, transport, topic, server, endpoint, serializer,
+                     exchange):
             self.controller = ServerSetupMixin.ServerController()
-            target = oslo_messaging.Target(topic=topic, server=server)
+            target = oslo_messaging.Target(topic=topic, server=server,
+                                           exchange=exchange)
             self.server = oslo_messaging.get_rpc_server(transport,
                                                         target,
                                                         [endpoint,
@@ -59,7 +61,7 @@ class ServerSetupMixin(object):
 
     class ServerController(object):
         def __init__(self):
-            self.stopped = threading.Event()
+            self.stopped = eventletutils.Event()
 
         def stop(self, ctxt):
             self.stopped.set()
@@ -81,25 +83,25 @@ class ServerSetupMixin(object):
     def __init__(self):
         self.serializer = self.TestSerializer()
 
-    def _setup_server(self, transport, endpoint, topic=None, server=None):
+    def _setup_server(self, transport, endpoint, topic=None, server=None,
+                      exchange=None):
         server = self.Server(transport,
                              topic=topic or 'testtopic',
                              server=server or 'testserver',
                              endpoint=endpoint,
-                             serializer=self.serializer)
+                             serializer=self.serializer,
+                             exchange=exchange)
 
         server.start()
         return server
 
-    def _stop_server(self, client, server, topic=None):
-        if topic is not None:
-            client = client.prepare(topic=topic)
+    def _stop_server(self, client, server, topic=None, exchange=None):
         client.cast({}, 'stop')
         server.wait()
 
-    def _setup_client(self, transport, topic='testtopic'):
-        return oslo_messaging.RPCClient(transport,
-                                        oslo_messaging.Target(topic=topic),
+    def _setup_client(self, transport, topic='testtopic', exchange=None):
+        target = oslo_messaging.Target(topic=topic, exchange=exchange)
+        return oslo_messaging.RPCClient(transport, target=target,
                                         serializer=self.serializer)
 
 
@@ -111,53 +113,68 @@ class TestRPCServer(test_utils.BaseTestCase, ServerSetupMixin):
 
     def setUp(self):
         super(TestRPCServer, self).setUp(conf=cfg.ConfigOpts())
+        # FakeExchangeManager uses a class-level exchanges mapping; "reset" it
+        # before tests assert amount of items stored
+        self.useFixture(fixtures.MonkeyPatch(
+            'oslo_messaging._drivers.impl_fake.FakeExchangeManager._exchanges',
+            new_value={}))
 
-    @mock.patch('warnings.warn')
-    def test_constructor(self, warn):
+    def test_constructor(self):
         transport = oslo_messaging.get_rpc_transport(self.conf, url='fake:')
         target = oslo_messaging.Target(topic='foo', server='bar')
         endpoints = [object()]
         serializer = object()
         access_policy = dispatcher.DefaultRPCAccessPolicy
 
-        warnings.simplefilter("always", FutureWarning)
         server = oslo_messaging.get_rpc_server(transport,
                                                target,
                                                endpoints,
                                                serializer=serializer,
-                                               access_policy=access_policy)
+                                               access_policy=access_policy,
+                                               executor='threading')
         self.assertIs(server.conf, self.conf)
         self.assertIs(server.transport, transport)
         self.assertIsInstance(server.dispatcher, oslo_messaging.RPCDispatcher)
         self.assertIs(server.dispatcher.endpoints, endpoints)
         self.assertIs(server.dispatcher.serializer, serializer)
-        self.assertEqual('blocking', server.executor_type)
-        self.assertEqual([
-            mock.call("blocking executor is deprecated. Executor default will "
-                      "be removed. Use explicitly threading or eventlet "
-                      "instead in version 'pike' and will be removed in "
-                      "version 'rocky'",
-                      category=FutureWarning, stacklevel=3)
-        ], warn.mock_calls)
+        self.assertEqual('threading', server.executor_type)
 
-    @mock.patch('warnings.warn')
-    def test_constructor_without_explicit_RPCAccessPolicy(self, warn):
+    def test_constructor_with_eventlet_executor(self):
         transport = oslo_messaging.get_rpc_transport(self.conf, url='fake:')
         target = oslo_messaging.Target(topic='foo', server='bar')
         endpoints = [object()]
         serializer = object()
+        access_policy = dispatcher.DefaultRPCAccessPolicy
 
-        warnings.simplefilter("always", FutureWarning)
-        oslo_messaging.get_rpc_server(transport, target,
-                                      endpoints, serializer=serializer)
-        self.assertEqual([
-            mock.call(mock.ANY, category=FutureWarning, stacklevel=3),
-            mock.call("blocking executor is deprecated. Executor default will "
-                      "be removed. Use explicitly threading or eventlet "
-                      "instead in version 'pike' and will be removed in "
-                      "version 'rocky'",
-                      category=FutureWarning, stacklevel=3)
-        ], warn.mock_calls)
+        server = oslo_messaging.get_rpc_server(transport,
+                                               target,
+                                               endpoints,
+                                               serializer=serializer,
+                                               access_policy=access_policy,
+                                               executor='eventlet')
+        self.assertIs(server.conf, self.conf)
+        self.assertIs(server.transport, transport)
+        self.assertIsInstance(server.dispatcher, oslo_messaging.RPCDispatcher)
+        self.assertIs(server.dispatcher.endpoints, endpoints)
+        self.assertIs(server.dispatcher.serializer, serializer)
+        self.assertEqual('eventlet', server.executor_type)
+
+    def test_constructor_with_unrecognized_executor(self):
+        transport = oslo_messaging.get_rpc_transport(self.conf, url='fake:')
+        target = oslo_messaging.Target(topic='foo', server='bar')
+        endpoints = [object()]
+        serializer = object()
+        access_policy = dispatcher.DefaultRPCAccessPolicy
+
+        self.assertRaises(
+            server_module.ExecutorLoadFailure,
+            oslo_messaging.get_rpc_server,
+            transport=transport,
+            target=target,
+            endpoints=endpoints,
+            serializer=serializer,
+            access_policy=access_policy,
+            executor='boom')
 
     def test_server_wait_method(self):
         transport = oslo_messaging.get_rpc_transport(self.conf, url='fake:')
@@ -300,14 +317,20 @@ class TestRPCServer(test_utils.BaseTestCase, ServerSetupMixin):
         self.assertEqual(['dsfoo', 'dsbar'], endpoint.pings)
 
     def test_call(self):
-        transport = oslo_messaging.get_rpc_transport(self.conf, url='fake:')
+        # NOTE(milan): using a separate transport instance for each the client
+        # and the server to be able to check independent transport instances
+        # can communicate over same exchange&topic
+        transport_srv = oslo_messaging.get_rpc_transport(self.conf,
+                                                         url='fake:')
+        transport_cli = oslo_messaging.get_rpc_transport(self.conf,
+                                                         url='fake:')
 
         class TestEndpoint(object):
             def ping(self, ctxt, arg):
                 return arg
 
-        server_thread = self._setup_server(transport, TestEndpoint())
-        client = self._setup_client(transport)
+        server_thread = self._setup_server(transport_srv, TestEndpoint())
+        client = self._setup_client(transport_cli)
 
         self.assertIsNone(client.call({}, 'ping', arg=None))
         self.assertEqual(0, client.call({}, 'ping', arg=0))
@@ -436,6 +459,20 @@ class TestRPCServer(test_utils.BaseTestCase, ServerSetupMixin):
 
         self._stop_server(client, server_thread)
 
+    @mock.patch('oslo_messaging.rpc.server.LOG')
+    def test_warning_when_notifier_transport(self, log):
+        transport = oslo_messaging.get_notification_transport(self.conf)
+        target = oslo_messaging.Target(topic='foo', server='bar')
+        endpoints = [object()]
+        serializer = object()
+
+        oslo_messaging.get_rpc_server(transport, target,
+                                      endpoints, serializer=serializer)
+        log.warning.assert_called_once_with(
+            "Using notification transport for RPC. Please use "
+            "get_rpc_transport to obtain an RPC transport "
+            "instance.")
+
 
 class TestMultipleServers(test_utils.BaseTestCase, ServerSetupMixin):
 
@@ -498,8 +535,8 @@ class TestMultipleServers(test_utils.BaseTestCase, ServerSetupMixin):
             single_server = params['server1'] == params['server2']
             return not (single_topic and single_server)
 
-        # fanout to multiple servers on same topic and exchange
-        # each endpoint will receive both messages
+        # fanout to multiple servers on same topic and exchange each endpoint
+        # will receive both messages
         def fanout_to_servers(scenario):
             params = scenario[1]
             fanout = params['fanout1'] or params['fanout2']
@@ -536,14 +573,16 @@ class TestMultipleServers(test_utils.BaseTestCase, ServerSetupMixin):
 
     def setUp(self):
         super(TestMultipleServers, self).setUp(conf=cfg.ConfigOpts())
+        self.useFixture(fixtures.MonkeyPatch(
+            'oslo_messaging._drivers.impl_fake.FakeExchangeManager._exchanges',
+            new_value={}))
 
     def test_multiple_servers(self):
-        url1 = 'fake:///' + (self.exchange1 or '')
-        url2 = 'fake:///' + (self.exchange2 or '')
-
-        transport1 = oslo_messaging.get_rpc_transport(self.conf, url=url1)
-        if url1 != url2:
-            transport2 = oslo_messaging.get_rpc_transport(self.conf, url=url1)
+        transport1 = oslo_messaging.get_rpc_transport(self.conf,
+                                                      url='fake:')
+        if self.exchange1 != self.exchange2:
+            transport2 = oslo_messaging.get_rpc_transport(self.conf,
+                                                          url='fake:')
         else:
             transport2 = transport1
 
@@ -563,12 +602,18 @@ class TestMultipleServers(test_utils.BaseTestCase, ServerSetupMixin):
             endpoint1 = endpoint2 = TestEndpoint()
 
         server1 = self._setup_server(transport1, endpoint1,
-                                     topic=self.topic1, server=self.server1)
+                                     topic=self.topic1,
+                                     exchange=self.exchange1,
+                                     server=self.server1)
         server2 = self._setup_server(transport2, endpoint2,
-                                     topic=self.topic2, server=self.server2)
+                                     topic=self.topic2,
+                                     exchange=self.exchange2,
+                                     server=self.server2)
 
-        client1 = self._setup_client(transport1, topic=self.topic1)
-        client2 = self._setup_client(transport2, topic=self.topic2)
+        client1 = self._setup_client(transport1, topic=self.topic1,
+                                     exchange=self.exchange1)
+        client2 = self._setup_client(transport2, topic=self.topic2,
+                                     exchange=self.exchange2)
 
         client1 = client1.prepare(server=self.server1)
         client2 = client2.prepare(server=self.server2)
@@ -584,9 +629,9 @@ class TestMultipleServers(test_utils.BaseTestCase, ServerSetupMixin):
         (client2.call if self.call2 else client2.cast)({}, 'ping', arg='2')
 
         self._stop_server(client1.prepare(fanout=None),
-                          server1, topic=self.topic1)
+                          server1, topic=self.topic1, exchange=self.exchange1)
         self._stop_server(client2.prepare(fanout=None),
-                          server2, topic=self.topic2)
+                          server2, topic=self.topic2, exchange=self.exchange2)
 
         def check(pings, expect):
             self.assertEqual(len(expect), len(pings))
@@ -670,11 +715,11 @@ class TestServerLocking(test_utils.BaseTestCase):
         # Test that if 2 threads call a method simultaneously, both will wait,
         # but only 1 will call the underlying executor method.
 
-        start_event = threading.Event()
-        finish_event = threading.Event()
+        start_event = eventletutils.Event()
+        finish_event = eventletutils.Event()
 
-        running_event = threading.Event()
-        done_event = threading.Event()
+        running_event = eventletutils.Event()
+        done_event = eventletutils.Event()
 
         _runner = [None]
 
@@ -700,7 +745,7 @@ class TestServerLocking(test_utils.BaseTestCase):
         runner = _runner[0]
         waiter = start2 if runner == start1 else start2
 
-        waiter_finished = threading.Event()
+        waiter_finished = eventletutils.Event()
         waiter.link(lambda _: waiter_finished.set())
 
         # At this point, runner is running start(), and waiter() is waiting for
@@ -749,8 +794,8 @@ class TestServerLocking(test_utils.BaseTestCase):
         # Ensure that if 2 threads wait for the completion of 'start', the
         # first will wait until complete_event is signalled, but the second
         # will continue
-        complete_event = threading.Event()
-        complete_waiting_callback = threading.Event()
+        complete_event = eventletutils.Event()
+        complete_waiting_callback = eventletutils.Event()
 
         start_state = self.server._states['start']
         old_wait_for_completion = start_state.wait_for_completion
@@ -767,7 +812,7 @@ class TestServerLocking(test_utils.BaseTestCase):
 
         # thread1 will wait for start to complete until we signal it
         thread1 = eventlet.spawn(self.server.stop)
-        thread1_finished = threading.Event()
+        thread1_finished = eventletutils.Event()
         thread1.link(lambda _: thread1_finished.set())
 
         self.server.start()
@@ -813,7 +858,7 @@ class TestServerLocking(test_utils.BaseTestCase):
         # Test that we generate a log message if we wait longer than
         # DEFAULT_LOG_AFTER
 
-        log_event = threading.Event()
+        log_event = eventletutils.Event()
         mock_log.warning.side_effect = lambda _, __: log_event.set()
 
         # Call stop without calling start. We should log a wait after 1 second
@@ -829,7 +874,7 @@ class TestServerLocking(test_utils.BaseTestCase):
         # Test that we generate a log message if we wait longer than
         # the number of seconds passed to log_after
 
-        log_event = threading.Event()
+        log_event = eventletutils.Event()
         mock_log.warning.side_effect = lambda _, __: log_event.set()
 
         # Call stop without calling start. We should log a wait after 1 second
@@ -845,7 +890,7 @@ class TestServerLocking(test_utils.BaseTestCase):
         # Test that we log a message after log_after seconds if we've also
         # specified an absolute timeout
 
-        log_event = threading.Event()
+        log_event = eventletutils.Event()
         mock_log.warning.side_effect = lambda _, __: log_event.set()
 
         # Call stop without calling start. We should log a wait after 1 second
@@ -870,7 +915,7 @@ class TestServerLocking(test_utils.BaseTestCase):
         # Start the server, which will also instantiate an executor
         self.server.start()
         self.server.stop()
-        shutdown_called = threading.Event()
+        shutdown_called = eventletutils.Event()
 
         # Patch the executor's stop method to be very slow
         def slow_shutdown(wait):
