@@ -4,31 +4,27 @@
 # it sets up the test system as needed.
 # Developer should setup their test systems in a similar way.
 
-# This setup needs to be run by a user that can run sudo.
+# This setup for amqp1 needs to be run by a user that can run sudo.
 
 function is_fedora {
     [ -f /usr/bin/yum ] && cat /etc/*release | grep -q -e "Fedora"
 }
 
-# Looks like /home/jenkins/worspace/gate-oslo.messaging-tox-py27-func-rabbit-ubuntu-xenial
-JOB_NAME=${WORKSPACE##*/}
-
-if [ "${JOB_NAME//gate-oslo.messaging-tox-py*-func-/}" == "${JOB_NAME}" ]; then
-    # not a functional test
-    exit 0
-fi
-
 # NOTE(sileht): we create the virtualenv only and use bindep directly
 # because tox doesn't have a quiet option...
 tox -ebindep --notest
 
+# TODO(ansmith) for now setup amqp1 dependencies for any profile.
+# Fix this when test-setup is passed environment profile setting.
+
 # NOTE(sileht): bindep return 1 if some packages have to be installed
-BINDEP_PROFILE=$(echo $JOB_NAME | cut -d- -f6)
-PACKAGES=$(.tox/bindep/bin/bindep -b -f bindep.txt $BINDEP_PROFILE || true)
+PACKAGES="$(.tox/bindep/bin/bindep -b -f bindep.txt amqp1 || true)"
+
+[ -n "$PACKAGES" ] || exit 0
 
 # inspired from project-config install-distro-packages.sh
 if apt-get -v >/dev/null 2>&1 ; then
-    [ $BINDEP_PROFILE == amqp1 ] && sudo add-apt-repository -y ppa:qpid/testing
+    sudo add-apt-repository -y ppa:qpid/testing
     sudo apt-get -qq update
     sudo PATH=/usr/sbin:/sbin:$PATH DEBIAN_FRONTEND=noninteractive \
         apt-get -q --option "Dpkg::Options::=--force-confold" \
